﻿// Network Table values:
//
//      thunderdashboard_elevator            number    elevator's height location/position (0 - 100)
//      thunderdashboard_elevator_target     number    the elevator location it is trying to reach (0 - 100)
//      elevator_target_position             number    the 'name' of the location the elevator is trying to reach
//      gamepiece_mode                       string    mode robot thinks it is in (hatch or cargo)
//      gamepiece_target_mode                string    mode robot wants to go to (hatch or cargo)
//      thunderdashboard_airpressure         number    PSI of the air system (0 - 125)
//      thunderdashboard_airpressure_lower   number    PSI lower than this means 'red' (0 - 125)
//      thunderdashboard_airpressure_upper   number    PSI lower than this (but above 'lower') means 'yellow' (0 - 125)
//      robot_match_time_remaining           number    second left in current match period (0 - 135)
//      thunderdashboard_match_period_length number    total seconds in current match period (0 - 135)


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
    public partial class Form2019 : FormBase
    {
        public Form2019()
        {
            InitializeComponent();

            Text = "Jupiter"; // Robot name goes here (i.e. tab's text)
            DebugTitle = "Josh and Ishan's Savior"; // title for the debug tab

            // load controller images
            ControllerAux = LoadImage("2019", "ControlsAux.png");
            ControllerDriver = LoadImage("2019", "ControlsDriver.png");
            GamePieceHatch = LoadImage("2019", "Hatch.png");
            GamePieceCargo = LoadImage("2019", "Cargo.png");

            // optional tabs
            TabLightsPsoc42xx = true;   // show the light tab that talks to the PSOC 42xx via a COM port?
            TabChart = false;            // show the chart tab?



            // *******************************************************************************
            // any custom control settings for this year's robot
            autoDelayWritten = false;       // we did not intialize the dashboard with a value yet
            //textBoxAutoDelay.Text = "0";    // default to 0 seconds
            //
            // *******************************************************************************



            // *******************************************************************************
            // NOTE: if a control/feature is not supported (i.e. control does not exist), then
            //       set the control to null (e.g. CameraFeed = null;)
            // *******************************************************************************

            // camera
            CameraPort = 5800; // port the camera feed is on (as of 2018 it is MPEJ on 1181, previous roboRIO years it was 1180)
            CameraFlip = true; // camera is mounted upside down, so flip image before showing it?
            CameraFeed = cameraControl; // control that gets the camera feed
            CameraCanChangeSettings = false; // can user change the image settings (e.g. DPI, FPS, brightness)
            CameraVisionDelegate = null; // method to call if there is any vision processing that the dashboard will do
            CameraFeedDirection = null; // indicates if camera feed is from the front or rear of robot (only needed if two cameras on robot)

            // drive
            //RobotDirection = labelReverseDrive; // will show REVERSE when driver controls have been swapped so rear of robot is the 'front'

            // auto mode
            //AutoModeComboBox = comboBoxAutoMode; // gets list of auto modes for user to select from
            //AutoModeDescriptionsTextBox = autoModeDescriptionsBox; // gets the description of the auto mode (not used in 2017)
            //AutoModeFromRobot = labelAutoModeValue; // auto mode the robot currently has selected

            // connection/status
            PanelConnected = panelConnected; // state of connection with network table
            StatusTextBox = statusTextBox; // any status messages about connection and images
        }

        // called with Reset button is pressed
        public override void buttonReset_Click(object sender, EventArgs e)
        {
            // just let base class to it all
            base.buttonReset_Click(sender, e);
        }

        // called when the network table is updated
        //
        // returns true if the form handled the key
        public override bool NetworkTableUpdated(string key, string value)
        {
            //int i;
            switch (key)
            {
                default:
                    // if we just connected, then set values that the dashboard drives
                    if (key == DashboardNetworkTable.Connected)
                    {
                        if (!autoDelayWritten)
                        {
                            autoDelayWritten = true;
                            //textBoxAutoDelay_TextChanged(null, null);// simulate user changing setting
                        }
                    }
                    return (base.NetworkTableUpdated(key, value));

                case "thunderdashboard_elevator":
                    meterElevator.Value = value;
                    break;

                case "thunderdashboard_elevator_target":
                    meterElevator.ValueTarget = value;
                    break;

                case "elevator_target_position":
                    labelElevatorTarget.Text = value;
                    break;

                case "gamepiece_mode":
                    if (value == "hatch")
                    {
                        _gamepieceIs = 0;
                    }
                    else if (value == "cargo")
                    {
                        _gamepieceIs = 1;
                    }
                    else
                    {
                        _gamepieceIs = -1;
                    }
                    GamePieceUpdate();
                    break;

                case "gamepiece_target_mode":
                    if (value == "hatch")
                    {
                        _gamepieceDesired = 0;
                        panelGamePiece.BackgroundImage = GamePieceHatch;
                    }
                    else if (value == "cargo")
                    {
                        _gamepieceDesired = 1;
                        panelGamePiece.BackgroundImage = GamePieceCargo;
                    }
                    else
                    {
                        _gamepieceDesired = -1;
                        panelGamePiece.BackgroundImage = null;
                    }
                    GamePieceUpdate();
                    break;

                case "thunderdashboard_airpressure":
                    textBoxPsi.Text = value;
                    _pressureCurrent = ValueToInt(value);
                    aGaugePsi.Value = _pressureCurrent;
                    PressureUpdate();
                    break;

                case "thunderdashboard_airpressure_lower":
                    _pressureLower = ValueToInt(value);
                    PressureUpdate();
                    PressureSetRanges();
                    break;

                case "thunderdashboard_airpressure_upper":
                    _pressureUpper = ValueToInt(value);
                    PressureUpdate();
                    PressureSetRanges();
                    break;

                case "robot_match_time_remaining":
                    _matchPeriodLeft = ValueToInt(value);
                    MatchTimeUpdate();
                    break;

                case "thunderdashboard_match_period_length":
                    _matchPeriodLength = ValueToInt(value);
                    MatchTimeUpdate();
                    break;

                //case "thunderdashboard_auto_start_delay":
                //    // if value is 0, but we are showing a blank, then do not
                //    // change the control; otherwise the user won't be able
                //    // to delete the enter and type a number
                //    if ((value == "0") && (textBoxAutoDelay.Text == ""))
                //    {
                //        break;
                //    }
                //    textBoxAutoDelay.Text = value;
                //    break;
            }

            return (false);
        }

        // convert string value, that is a number, to a number
        private int ValueToInt(string value)
        {
            try
            {
                return (Convert.ToInt32(value));
            }
            catch
            {
                return (0);
            }
        }

        // update the gauge ranges
        private void PressureSetRanges()
        {
            AGaugeRange agr;

            agr = aGaugePsi.GaugeRanges.FindByName("GaugeRangeRed");
            if (agr != null)
            {
                agr.StartValue = 0;
                agr.EndValue = _pressureLower;
            }
            agr = aGaugePsi.GaugeRanges.FindByName("GaugeRangeYellow");
            if (agr != null)
            {
                agr.StartValue = _pressureLower;
                agr.EndValue = _pressureUpper;
            }
        }

        // update background of pressure number
        private void PressureUpdate()
        {
            if (_pressureCurrent < _pressureLower)
            {
                textBoxPsi.BackColor = Color.Red;
                aGaugePsi.NeedleColor1 = AGaugeNeedleColor.Red;
                aGaugePsi.NeedleColor2 = Color.DarkRed;
            }
            else if (_pressureCurrent < _pressureUpper)
            {
                textBoxPsi.BackColor = Color.Yellow;
                aGaugePsi.NeedleColor1 = AGaugeNeedleColor.Yellow;
                aGaugePsi.NeedleColor2 = Color.DarkKhaki;
            }
            else
            {
                textBoxPsi.BackColor = SystemColors.Control;
                aGaugePsi.NeedleColor1 = AGaugeNeedleColor.Gray;
                aGaugePsi.NeedleColor2 = Color.DimGray;
            }
        }

        // game piece update
        private void GamePieceUpdate()
        {
            // see if everything is in sync
            if ((_gamepieceDesired < 0) || (_gamepieceIs < 0) || (_gamepieceDesired != _gamepieceIs))
            {
                panelGamePiece.BackColor = Color.Red;
            }
            else
            {
                panelGamePiece.BackColor = SystemColors.Control;
            }
        }

        private Image GetDigit(int digit)
        {
            switch (digit)
            {
                default: return (null);
                case 0: return (Properties.Resources.Digit0);
                case 1: return (Properties.Resources.Digit1);
                case 2: return (Properties.Resources.Digit2);
                case 3: return (Properties.Resources.Digit3);
                case 4: return (Properties.Resources.Digit4);
                case 5: return (Properties.Resources.Digit5);
                case 6: return (Properties.Resources.Digit6);
                case 7: return (Properties.Resources.Digit7);
                case 8: return (Properties.Resources.Digit8);
                case 9: return (Properties.Resources.Digit9);
            }
        }

        private void MatchTimeUpdate()
        {
            bool showSlider = false;

            // only show the slider if we were given a total match period
            // AND the length isn't too small
            if ((_matchPeriodLength != 0) && (_matchPeriodLength >= _matchPeriodLeft))
            {
                showSlider = true;
            }

            // if we are showing the slider, then set its info
            if (showSlider)
            {
                meterControlTime.MinMax(0, _matchPeriodLength);
                meterControlTime.Value = _matchPeriodLeft.ToString();
            }

            // now we can show/hide the slider
            meterControlTime.Visible = showSlider;

            // always update the digital display
            int minutes = (_matchPeriodLeft / 60);
            int seconds = (_matchPeriodLeft - (minutes * 60));
            int secondsDigit1 = (seconds / 10);
            int secondsDigit2 = (seconds - (secondsDigit1 * 10));
            pictureBoxTimeMin.Image = GetDigit(minutes);
            pictureBoxTimeSec1.Image = GetDigit(secondsDigit1);
            pictureBoxTimeSec2.Image = GetDigit(secondsDigit2);
            //textBoxTimeRemaining.Text = string.Format("{0}:{1:D2}", minutes, seconds);
        }

        //// called when the Auto Delay value is changed
        //private void textBoxAutoDelay_TextChanged(object sender, EventArgs e)
        //{
        //    string value = textBoxAutoDelay.Text;
        //    string validated = "";
        //    int lengthToCheck;

        //    // valid the length. while we set the control's property to prevent
        //    // the user from putting in too many characters, it could be too
        //    // long if network tables sent in too long of a value
        //    lengthToCheck = value.Length;
        //    if (value.Length > textBoxAutoDelay.MaxLength)
        //    {
        //        lengthToCheck = textBoxAutoDelay.MaxLength;
        //    }

        //    // make sure each character is a digit. if not, then put back just the digits
        //    for (int i = 0; i < lengthToCheck; i++)
        //    {
        //        if ((value[i] >= 0x30) && (value[i] <= 0x39))
        //        {
        //            validated += value[i];
        //        }
        //    }
        //    if (validated != value)
        //    {
        //        textBoxAutoDelay.Text = validated;
        //        textBoxAutoDelay.Select(textBoxAutoDelay.Text.Length, 0);
        //    }

        //    // see what the value is and give it to the dashboard
        //    int delay;
        //    try
        //    {
        //        if (textBoxAutoDelay.Text == "")
        //        {
        //            delay = 0;
        //        }
        //        else
        //        {
        //            delay = Convert.ToInt32(textBoxAutoDelay.Text);
        //        }
        //    }
        //    catch
        //    {
        //        delay = 0;
        //    }

        //    // if there is a delay, let's 'highlight' the auto mode selection so they see it is different
        //    if (delay == 0)
        //    {
        //        comboBoxAutoMode.FlatStyle = FlatStyle.Standard;
        //        comboBoxAutoMode.BackColor = SystemColors.Window;
        //    }
        //    else
        //    {
        //        comboBoxAutoMode.FlatStyle = FlatStyle.Flat;
        //        comboBoxAutoMode.BackColor = Color.Yellow;
        //    }

        //    // update robot
        //    DashboardNetworkTable.PutNumber("thunderdashboard_auto_start_delay", Convert.ToDouble(delay));
        //}

        // member variables
        private bool autoDelayWritten = false;  // did we give the dashboard the value yet?
        private int _pressureCurrent = 0; // current pressure
        private int _pressureLower = 0; // if pressure lower than this, then show red
        private int _pressureUpper = 0; // if pressure between this and _pressureLower, then show yellow
        private int _gamepieceDesired = -1; // what the driver wants: 0 - hatch; 1 - cargo; -1 - unknown
        private int _gamepieceIs = -1; // what the robot is: 0 - hatch; 1 - cargo; -1 - unknown
        private int _matchPeriodLength = 0; // total seconds in current match period
        private int _matchPeriodLeft = 0; // seconds left in the current match period
    }
}
