﻿// Network Table values:
//
//      thunderdashboard_wheel               number    what robot sees for the wheel color (0-don't know; 1-Red; 2-Green; 3-Blue; 4-Yellow)
//      thunderdashboard_wheel_end           number    color FMS is asking for in final stage (same as thunderdashboard_wheel)
//      thunderdashboard_shooter_speed       number    speed the shooter is running at (0 - 100)
//      thunderdashboard_shooter_pos         number    position of turret in degrees (0 - 360)
//      thunderdashboard_shooter_hood        number    position of the shooter's hood (0 - 100)
//      thunderdashboard_hang_pos            number    position of the arm (0 - 100)
//      thunderdashboard_hang_lock           number    is hang ratchet locked? (0-no; 1-yes)
//      thunderdashboard_intake_pos          number    is intake up? (0-no/down/deployed; 1-yes/up/retracted)
//      thunderdashboard_ballcount           number    number of balls in robot (-1-don't know; 0 - 5)
//      thunderdashboard_auto_start_delay    number    seconds, 0 to 15, to delay start of auto mode; force default to 0


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
    public partial class Form2020 : FormBase
    {
        public Form2020()
        {
            InitializeComponent();

            Text = "Darth Thunderous the Wise"; // Robot name goes here (i.e. tab's text)
            DebugTitle = "Josh and Ishan's Savior"; // title for the debug tab

            // load images
            ControllerAux = LoadImage("2020", "ControlsAux.png");
            ControllerDriver = LoadImage("2020", "ControlsDriver.png");
            IntakeUnknown = LoadImage("2020", "Intake_Unknown.png");
            IntakeUp = LoadImage("2020", "Intake_Up.png");
            IntakeDown = LoadImage("2020", "Intake_Down.png");
            WheelUnknown = LoadImage("2020", "Wheel_Unknown.png");
            WheelRed = LoadImage("2020", "Wheel_Red.png");
            WheelGreen = LoadImage("2020", "Wheel_Green.png");
            WheelBlue = LoadImage("2020", "Wheel_Blue.png");
            WheelYellow = LoadImage("2020", "Wheel_Yellow.png");

            // optional tabs
            TabLightsPsoc42xx = true;   // show the light tab that talks to the PSOC 42xx via a COM port?
            TabChart = true;            // show the chart tab?



            // *******************************************************************************
            // any custom control settings for this year's robot
            //autoDelayWritten = false;                // we did not intialize the dashboard with a value yet
            textBoxAutoDelay.Text = "0";             // default to 0 seconds
            comboBoxStartingCount.SelectedIndex = 0; // 0 balls in robot
            IntakePosition("-1");                    // set graphic to unknown
            WheelPosition("0");                      // set graphic to unknown
            //
            // *******************************************************************************



            // *******************************************************************************
            // NOTE: if a control/feature is not supported (i.e. control does not exist), then
            //       set the control to null (e.g. CameraFeed = null;)
            // *******************************************************************************

            // camera
            CameraPort = 1181; // port the camera feed is on (as of 2018 it is MPEJ on 1181, previous roboRIO years it was 1180)
            CameraStream = CameraControl.StreamType.Mjpeg; // type of video stream (2018 was Mjpeg, 2019 was FfMpeg, 2020 back to Mjpeg)
            CameraFlip = false; // camera is mounted upside down, so flip image before showing it?
            CameraFeed = cameraControl; // control that gets the camera feed
            CameraCanChangeSettings = false; // can user change the image settings (e.g. DPI, FPS, brightness)
            CameraVisionDelegate = null; // method to call if there is any vision processing that the dashboard will do
            CameraFeedDirection = null; // indicates if camera feed is from the front or rear of robot (only needed if two cameras on robot)
            CameraImage = LoadImage("2020", "Camera.png");
            //
            CameraIpLimeLight = null; // use the default which is: 10.TE.AM.11 (where TEAM is the currently set team number)
            CameraPortLimeLight = 5800; // port the camera feed is on
            CameraStreamLimeLight = CameraControl.StreamType.Mjpeg; // type of video stream
            CameraFeedLimeLight = cameraControlLimeLight; // control that gets the camera feed
            CameraImageLimeLight = LoadImage("2020", "CameraLimeLight.png");
            //
            CameraBothStreamsActive = false;


            // drive
            //RobotDirection = labelReverseDrive; // will show REVERSE when driver controls have been swapped so rear of robot is the 'front'
            DriveType = labelDriveType; // will show ARCADE is default is Tank, but currently in Arcade mode
            DriveTypeDefaultIsArcade = false;

            // auto mode
            AutoModeComboBox = comboBoxAutoMode; // gets list of auto modes for user to select from
            //AutoModeDescriptionsTextBox = autoModeDescriptionsBox; // gets the description of the auto mode (not used in 2017)
            //AutoModeFromRobot = labelAutoModeValue; // auto mode the robot currently has selected

            // connection/status
            PanelConnected = panelConnected; // state of connection with network table
            StatusTextBox = statusTextBox; // any status messages about connection and images
        }

        // called with Reset button is pressed
        public override void buttonReset_Click(object sender, EventArgs e)
        {
            // just let base class to it all
            base.buttonReset_Click(sender, e);
        }

        // called when the network table is updated
        //
        // returns true if the form handled the key
        public override bool NetworkTableUpdated(string key, string value)
        {
            switch (key)
            {
                default:
                    // if we just connected, then set values that the dashboard drives (besides Auto mode)
                    if (key == DashboardNetworkTable.Connected)
                    {
                        textBoxAutoDelay_TextChanged(null, null);// simulate user changing setting
                        comboBoxStartingCount_SelectedIndexChanged(null, null);// simulate user changing setting)

                        int delay = 0;
                        DashboardNetworkTable.PutNumber("r1", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r2", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r3", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r4", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r5", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r6", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r7", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r8", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r9", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r10", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r11", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r12", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r13", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r14", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r15", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r16", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r17", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r18", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r19", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r20", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r21", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r22", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r23", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r24", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r25", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r26", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r27", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r28", Convert.ToDouble(delay));
                        DashboardNetworkTable.PutNumber("r29", Convert.ToDouble(delay));
                    }
                    return (base.NetworkTableUpdated(key, value));

                case "thunderdashboard_wheel": // (0-don't know; 1-Red; 2-Green; 3-Blue; 4-Yellow)
                    WheelPosition(value);
                    break;

                case "thunderdashboard_wheel_end": // (same as thunderdashboard_wheel)
                    WheelBackground(value);
                    break;

                case "thunderdashboard_shooter_speed": // (0 - 100)
                    meterShooterSpeed.Value = value;
                    break;

                case "thunderdashboard_shooter_pos": // (0 - 360)
                    break;

                case "thunderdashboard_shooter_hood": // (0 - 100)
                    meterShooterHood.Value = value;
                    break;

                case "thunderdashboard_hang_pos": // (0 - 100)
                    meterHang.Value = value;
                    break;

                case "thunderdashboard_hang_lock": // (0-no; 1-yes)
                    meterHang.Color((value == "0") ? Color.Lime : Color.Red);
                    labelHang.ForeColor = ((value == "0") ? SystemColors.ControlText : Color.Red); // set in case meter is at 0
                    break;

                case "thunderdashboard_intake_pos": // (0-no/down/deployed; 1-yes/up/retracted)
                    IntakePosition(value);
                    break;

                case "thunderdashboard_ballcount": // (-1-don't know; 0 - 5)
                    BallCount(value);
                    break;

                case "thunderdashboard_auto_start_delay":
                    // if value is 0, but we are showing a blank, then do not
                    // change the control; otherwise the user won't be able
                    // to delete the entry and type a number
                    if ((value == "0") && (textBoxAutoDelay.Text == ""))
                    {
                        break;
                    }
                    textBoxAutoDelay.Text = value;
                    break;
            }

            return (false);
        }

        // convert string value, that is a number, to a number
        private int ValueToInt(string value)
        {
            try
            {
                return (Convert.ToInt32(value));
            }
            catch
            {
                return (0);
            }
        }

        // called when the Auto Delay value is changed
        private void textBoxAutoDelay_TextChanged(object sender, EventArgs e)
        {
            string value = textBoxAutoDelay.Text;
            string validated = "";
            int lengthToCheck;

            // valid the length. while we set the control's property to prevent
            // the user from putting in too many characters, it could be too
            // long if network tables sent in too long of a value
            lengthToCheck = value.Length;
            if (value.Length > textBoxAutoDelay.MaxLength)
            {
                lengthToCheck = textBoxAutoDelay.MaxLength;
            }

            // make sure each character is a digit. if not, then put back just the digits
            for (int i = 0; i < lengthToCheck; i++)
            {
                if ((value[i] >= 0x30) && (value[i] <= 0x39))
                {
                    validated += value[i];
                }
            }
            if (validated != value)
            {
                textBoxAutoDelay.Text = validated;
                textBoxAutoDelay.Select(textBoxAutoDelay.Text.Length, 0);
            }

            // see what the value is and give it to the dashboard
            int delay;
            try
            {
                if (textBoxAutoDelay.Text == "")
                {
                    delay = 0;
                }
                else
                {
                    delay = Convert.ToInt32(textBoxAutoDelay.Text);
                }
            }
            catch
            {
                delay = 0;
            }

            // if there is a delay, let's 'highlight' the auto mode selection so they see it is different
            if (delay == 0)
            {
                comboBoxAutoMode.FlatStyle = FlatStyle.Standard;
                comboBoxAutoMode.BackColor = SystemColors.Window;
            }
            else
            {
                comboBoxAutoMode.FlatStyle = FlatStyle.Flat;
                comboBoxAutoMode.BackColor = Color.Yellow;
            }

            // update robot
            DashboardNetworkTable.PutNumber("thunderdashboard_auto_start_delay", Convert.ToDouble(delay));
        }

        // user changed the starting ball count
        private void comboBoxStartingCount_SelectedIndexChanged(object sender, EventArgs e)
        {
            int count = FormMain.NumberFromCombobox(comboBoxStartingCount, 0);
            DashboardNetworkTable.PutNumber("thunderdashboard_starting_count", Convert.ToDouble(count));
        }

        // update ball count item
        private void BallCount(string value)
        {
            int balls = ValueToInt(value);
            labelBallCount.Text = value;
            if (balls < 0)
            {
                labelBallCount.Text = "?";                  // don't know how many balls we have
                labelBallCount.BackColor = SystemColors.Control;
            }
            else if ((balls >= 0) && (balls <= 3))
            {
                labelBallCount.BackColor = Color.Green;     // still room for balls
            }
            else if ((balls >= 4) && (balls <= 4))
            {
                labelBallCount.BackColor = Color.Yellow;    // only room for one more ball
            }
            else
            {
                labelBallCount.BackColor = Color.Red;       // have maximum number of balls
            }
        }

        // update intake graphic
        private void IntakePosition(string value)
        {
            switch (ValueToInt(value))
            {
                default: // unknown
                    panelIntake.BackgroundImage = IntakeUnknown;
                    break;
                case 0: // down 
                    panelIntake.BackgroundImage = IntakeDown;
                    break;
                case 1: // up
                    panelIntake.BackgroundImage = IntakeUp;
                    break;
            }
        }

        // update wheel graphic
        private void WheelPosition(string value)
        {
            switch (ValueToInt(value))
            {
                default:
                case 0: // unknown
                    panelWheel.BackgroundImage = WheelUnknown;
                    break;
                case 1: // Red
                    panelWheel.BackgroundImage = WheelRed;
                    break;
                case 2: // Green
                    panelWheel.BackgroundImage = WheelGreen;
                    break;
                case 3: // Blue
                    panelWheel.BackgroundImage = WheelBlue;
                    break;
                case 4: // Yellow
                    panelWheel.BackgroundImage = WheelYellow;
                    break;
            }
        }

        // update wheel color for final stage
        private void WheelBackground(string value)
        {
            switch (ValueToInt(value))
            {
                default:
                case 0: // unknown
                    panelWheel.BackColor = SystemColors.Control;
                    break;
                case 1: // Red
                    panelWheel.BackColor = Color.Red;
                    break;
                case 2: // Green
                    panelWheel.BackColor = Color.Lime;
                    break;
                case 3: // Blue
                    panelWheel.BackColor = Color.Cyan;
                    break;
                case 4: // Yellow
                    panelWheel.BackColor = Color.Yellow;
                    break;
            }
        }


        // Intake bitmaps
        public Image IntakeUnknown { get { return (_intakeUnknown); } set { _intakeUnknown = value; } }
        private Image _intakeUnknown = null;
        public Image IntakeUp { get { return (_intakeUp); } set { _intakeUp = value; } }
        private Image _intakeUp = null;
        public Image IntakeDown { get { return (_intakeDown); } set { _intakeDown = value; } }
        private Image _intakeDown = null;

        // Wheel bitmaps
        public Image WheelUnknown { get { return (_wheelUnknown); } set { _wheelUnknown = value; } }
        private Image _wheelUnknown = null;
        public Image WheelRed { get { return (_wheelRed); } set { _wheelRed = value; } }
        private Image _wheelRed = null;
        public Image WheelGreen { get { return (_wheelGreen); } set { _wheelGreen = value; } }
        private Image _wheelGreen = null;
        public Image WheelBlue { get { return (_wheelBlue); } set { _wheelBlue = value; } }
        private Image _wheelBlue = null;
        public Image WheelYellow { get { return (_wheelYellow); } set { _wheelYellow = value; } }
        private Image _wheelYellow = null;

        // member variables
        //private bool autoDelayWritten = false;  // did we give the dashboard the value yet?
    }
}
