﻿// handle MJPEG video streaming cameras

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Threading;
using System.Net;
using System.Collections;
using System.IO;
using System.Drawing;
using System.ComponentModel;
//using thundervideo;
using System.Reflection;

namespace ThunderDashboard
{
    partial class CameraControl : PictureBox
    {
        // runs the camera (for background worker) until told to exit camera
        private void RunCameraFfmpej()
        {
            ThunderVideoError tve;

            // if we cannot load the DLL, then just bail
            if (!LoadThunderVision())
            {
                return;
            }

            // get connection to the robot stream
            while (!_shutdownCamera && !_resetCamera)
            {
                // Connect to remote camera.
                try
                {
                    // need to give the robot time to open a camera link for us, and give us a response
                    tve = (ThunderVideoError)_ffmpegOpen.Invoke(_ffmpegThundervideo, new object[] { _port, 2000 });
                    if (tve != ThunderVideoError.OK)
                    {
                        // robot didn't have any type of response, so it must not be fully ready
                        UpdateStatusDescription("At " + DateTime.Now + " open failed: " + tve.ToString());
                        return;
                    }
                    _ffmpegOpened = true;
                    break; // we are connected
                }
                catch (Exception e)
                {
                    UpdateStatusDescription("At " + DateTime.Now + " Failed while connecting to camera: " + e.ToString());
                    return;
                }
            }
            if (_shutdownCamera || _resetCamera)
            {
                // we broke out of above while loop because we need to restart/shutdown, so get out of here
                if (_ffmpegOpened)
                {
                    _ffmpegClose.Invoke(_ffmpegThundervideo, new object[] { });
                    _ffmpegOpened = false;
                }
                if (_shutdownCamera)
                {
                    CloseCameraFfmpej();
                }
                return;
            }

            // now that the stream is open, get each image until we are told to shutdown/restart
            try
            {
                _totalInBuffer = 0;
                while (!_shutdownCamera && !_resetCamera)
                {
                    // get a frame
                    object[] args = new object[] { null, 500 };
                    tve = (ThunderVideoError)_ffmpegGetNextFrame.Invoke(_ffmpegThundervideo, args);
                    if (tve != ThunderVideoError.OK)
                    {
                        UpdateStatusDescription("At " + DateTime.Now + " image error " + tve.ToString());
                        break;
                    }

                    // get bitmap to send to UI
                    UpdateStatusDescription("At " + DateTime.Now + " got image");

                    // make sure the image is updated in the UI thread
                    Invoke(new UpdateImageDelegate(UpdateImage), new Bitmap((Bitmap)args[0]));
                }
            }
            catch (Exception e)
            {
                UpdateStatusDescription("At " + DateTime.Now + " Failed reading images: " + e.ToString());
            }

            // make sure the stream is closed
            if (_ffmpegOpened)
            {
                _ffmpegClose.Invoke(_ffmpegThundervideo, new object[] { });
                _ffmpegOpened = false;
            }
        }


        private void CloseCameraFfmpej()
        {
            if (_ffmpegOpened)
            {
                _ffmpegOpened = false;
                if (_ffmpegClose != null)
                {
                    _ffmpegClose.Invoke(_ffmpegThundervideo, new object[] { });
                }
            }
            _ffmpegThundervideo = null;
            _ffmpegAssembly = null;
            if (_ffmpegDomain != null)
            {
                AppDomain.Unload(_ffmpegDomain);
                _ffmpegDomain = null;
            }
            _ffmpegOpen = null;
            _ffmpegClose = null;
            _ffmpegGetNextFrame = null;
        }

        private bool LoadThunderVision()
        {
            // do NOT do this again
            if (_ffmpegDomain != null)
            {
                return (true);
            }

            // confirm the DLL is there
            string file = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            string dll = "thundervideo.dll";
            file = Path.Combine(file, dll);
            if (!File.Exists(file))
            {
                UpdateStatusDescription("At " + DateTime.Now + " " + file + " not found");
                return (false);
            }

            try
            {
                // create a domain to run the assembly in
                _ffmpegDomain = AppDomain.CreateDomain("thundervideo");
                if (_ffmpegDomain == null)
                {
                    UpdateStatusDescription("At " + DateTime.Now + " unable to create domain");
                    return (false);
                }

                // load the assembly into the domain
                _ffmpegAssembly = _ffmpegDomain.Load(new AssemblyName() { CodeBase = file });
                if (_ffmpegAssembly == null)
                {
                    AppDomain.Unload(_ffmpegDomain);
                    _ffmpegDomain = null;
                    UpdateStatusDescription("At " + DateTime.Now + " unable to load domain");
                    return (false);
                }

                // get an instance of the object
                _ffmpegThundervideo = _ffmpegAssembly.CreateInstance("thundervideo.ThunderVideo");
                if (_ffmpegThundervideo == null)
                {
                    _ffmpegAssembly = null;
                    AppDomain.Unload(_ffmpegDomain);
                    _ffmpegDomain = null;
                    UpdateStatusDescription("At " + DateTime.Now + " unable to create object");
                    return (false);
                }

                // load the methods
                _ffmpegOpen = _ffmpegThundervideo.GetType().GetMethod("open");
                _ffmpegClose = _ffmpegThundervideo.GetType().GetMethod("close");
                _ffmpegGetNextFrame = _ffmpegThundervideo.GetType().GetMethod("getNextFrame");
            }
            catch (Exception e)
            {
                UpdateStatusDescription("At " + DateTime.Now + " Failed while loading " + dll + ": " + e.ToString());
                return (false);
            }

            return (true);
        }



        // member variables
        private enum ThunderVideoError { OK, FAILED_TIMEOUT, FAILED_OTHER };
        private bool _ffmpegOpened = false;
        private AppDomain _ffmpegDomain = null;
        private Assembly _ffmpegAssembly = null;
        private object _ffmpegThundervideo = null;
        private MethodInfo _ffmpegOpen = null;
        private MethodInfo _ffmpegClose = null;
        private MethodInfo _ffmpegGetNextFrame = null;
    }
}
