﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
    public partial class FormBase : Form
    {
        public FormBase()
        {
        }

        public void Start()
        {
            // hook up with the new autonomous controls
            _autoMode.Initialize(AutoModeComboBox, AutoModeDescriptionsTextBox, _autoModeKey, _keyPrefixAuto);    // tell object which controls to use

            // if we have both camera feeds, then show main camera feed for now
            if ((CameraFeed != null) && (CameraFeedLimeLight != null))
            {
                CameraFeedLimeLight.Visible = false;
            }
            if (CameraFeed != null)
            {
                CameraFeed.MouseDoubleClick += CameraFeed_MouseDoubleClick;
            }
            if (CameraFeedLimeLight != null)
            {
                CameraFeedLimeLight.MouseDoubleClick += CameraFeedLimeLight_MouseDoubleClick;
            }

            // start camera. also do LimeLight if allowed to have both cameras running
            StartCameras(true, CameraBothStreamsActive);
        }

        private void CameraFeed_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (CameraFeedLimeLight != null)
            {
                CameraFeed.Visible = false;
                CameraFeedLimeLight.Visible = true;
                StartCameras(false, true);
            }
        }

        private void CameraFeedLimeLight_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (CameraFeed != null)
            {
                CameraFeed.Visible = true;
                CameraFeedLimeLight.Visible = false;
                StartCameras(true, false);
            }
        }

        public void Stop()
        {
            // if we have a camera, stop it
            if (CameraFeed != null)
            {
                CameraFeed.Stop();
            }
            if (CameraFeedLimeLight != null)
            {
                CameraFeedLimeLight.Stop();
            }
        }

        // Team number
        public string TeamNumber { get { return (_teamNumber); } set { _teamNumber = value; } }
        private string _teamNumber = "1511";

        // Robot IP address
        public string RobotIpAddress { get { return ((_robotIpAddress == null) ? "roboRIO-" + _teamNumber + "-FRC.local" : _robotIpAddress); } set { _robotIpAddress = value; } }
        private string _robotIpAddress = null;//"127.0.0.1";

        // Image of the driver's controller 
        public Image ControllerDriver { get { return (_controllerDriver); } set { _controllerDriver = value; } }
        private Image _controllerDriver = null;

        // Image of the aux's controller 
        public Image ControllerAux { get { return (_controllerAux); } set { _controllerAux = value; } }
        private Image _controllerAux = null;

        // Image of the hatch game piece
        public Image GamePieceHatch { get { return (_gamePieceHatch); } set { _gamePieceHatch = value; } }
        private Image _gamePieceHatch = null;

        // Image of the cargo game piece
        public Image GamePieceCargo { get { return (_gamePieceCargo); } set { _gamePieceCargo = value; } }
        private Image _gamePieceCargo = null;

        // Debug tab title
        public string DebugTitle { get { return (_debugTitle); } set { _debugTitle = value; } }
        private string _debugTitle = "Debug";

        // optionally show Light tab
        public bool TabLightsPsoc42xx { get { return (_tabLightsPsoc42xx); } set { _tabLightsPsoc42xx = value; } }
        private bool _tabLightsPsoc42xx = true;

        // optionally show Chart tab
        public bool TabChart { get { return (_tabChart); } set { _tabChart = value; } }
        private bool _tabChart = true;

        // Camera's port
        public int CameraPort { get { return (_cameraPort); } set { _cameraPort = value; } }
        private int _cameraPort = 1181;

        // Camera's stream type
        public CameraControl.StreamType CameraStream { get { return (_cameraStream); } set { _cameraStream = value; } }
        private CameraControl.StreamType _cameraStream = CameraControl.StreamType.UsePort;

        // Camera's initial image
        public Image CameraImage { get { return (_cameraImage); } set { _cameraImage = value; } }
        private Image _cameraImage = null;

        // Flip camera image?
        public bool CameraFlip { get { return (_cameraFlip); } set { _cameraFlip = value; } }
        private bool _cameraFlip = false;

        // control for Camera's
        public CameraControl CameraFeed { get { return (_cameraControl); } set { _cameraControl = value; } }
        private CameraControl _cameraControl = null;

        // control for LimeLight Camera
        public CameraControl CameraFeedLimeLight { get { return (_cameraControlLimeLight); } set { _cameraControlLimeLight = value; } }
        private CameraControl _cameraControlLimeLight = null;

        // LimeLight Camera's port
        public int CameraPortLimeLight { get { return (_cameraPortLimeLight); } set { _cameraPortLimeLight = value; } }
        private int _cameraPortLimeLight = 5800;

        // LimeLight Camera's IP
        public string CameraIpLimeLight { get { return (((_cameraIpLimeLight == null) ? ("10." + _teamNumber.Substring(0, 2) + "." + _teamNumber.Substring(2, 2) + ".11") : _cameraIpLimeLight)); } set { _cameraIpLimeLight = value; } }
        private string _cameraIpLimeLight = "";

        // LimeLight Camera's stream type
        public CameraControl.StreamType CameraStreamLimeLight { get { return (_cameraStreamLimeLight); } set { _cameraStreamLimeLight = value; } }
        private CameraControl.StreamType _cameraStreamLimeLight = CameraControl.StreamType.Mjpeg;

        // LimeLight Camera's initial image
        public Image CameraImageLimeLight { get { return (_cameraImageLimeLight); } set { _cameraImageLimeLight = value; } }
        private Image _cameraImageLimeLight = null;

        public bool CameraBothStreamsActive { get { return (_cameraBothStreamsActive); } set { _cameraBothStreamsActive = value; } }
        private bool _cameraBothStreamsActive = false;

        // Can change image settings?
        public bool CameraCanChangeSettings { get { return (_cameraCanChangeSettings); } set { _cameraCanChangeSettings = value; } }
        private bool _cameraCanChangeSettings = false;

        public CameraControl.VisionDelegate CameraVisionDelegate { get { return (_cameraVisionDelegate); } set { _cameraVisionDelegate = value; } }
        private CameraControl.VisionDelegate _cameraVisionDelegate = null;

        // control for Auto mode from robot
        public Label AutoModeFromRobot { get { return (_labelAutoModeValue); } set { _labelAutoModeValue = value; } }
        private Label _labelAutoModeValue = null;

        // textbox control that is used to show ant status messages
        public ComboBox AutoModeComboBox { get { return (_comboBoxAutoMode); } set { _comboBoxAutoMode = value; } }
        private ComboBox _comboBoxAutoMode = null;

        // textbox control that shows the description of the selected auto mode
        public TextBox AutoModeDescriptionsTextBox { get { return (_autoModeDescriptionsBox); } set { _autoModeDescriptionsBox = value; } }
        private TextBox _autoModeDescriptionsBox = null;

        // control to says which side the camera feed is coming from
        public Label CameraFeedDirection { get { return (_labelCameraSide); } set { _labelCameraSide = value; } }
        private Label _labelCameraSide = null;

        // control to says which way the robot is being driven (i.e. which end if the front)
        public Label RobotDirection { get { return (_labelReverseDrive); } set { _labelReverseDrive = value; } }
        private Label _labelReverseDrive = null;

        // control to says which tpe of driver style is being used
        public Label DriveType { get { return (_labelDriveType); } set { _labelDriveType = value; UpdateDriveType(); } }
        private Label _labelDriveType = null;

        // which mode is the default for that year
        public bool DriveTypeDefaultIsArcade { get { return (_isDriveTypeDefaultArcade); } set { _isDriveTypeDefaultArcade = value; UpdateDriveType(); } }
        private bool _isDriveTypeDefaultArcade = true;

        // panel that shows robot is connected or not
        public Panel PanelConnected { get { return (_panelConnected); } set { _panelConnected = value; } }
        private Panel _panelConnected = null;

        // textbox control that is used to show ant status messages
        public TextBox StatusTextBox { get { return (_statusTextBox); } set { _statusTextBox = value; } }
        private TextBox _statusTextBox = null;

        // load an embedded resource
        protected Image LoadImage(string year, string filename)
        {
            try
            {
                System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
                System.IO.Stream imageStream = assembly.GetManifestResourceStream("ThunderDashboard._" + year + "." + filename);
                return (new Bitmap(imageStream));
            }
            catch
            {
                MessageBox.Show("Error finding image: " + year + "\\" + filename + "\n\nMake sure its 'Build Action' property is set to 'Embedded Resource'.");
            }
            return (null);
        }

        // called when the network table is updated
        //
        // returns true if the form handled the key
        public virtual bool NetworkTableUpdated(string key, string value)
        {
            // see which item it is in the network table that we have
            switch (key)
            {
                default:
                    // see if something we need to send to auto
                    if (key.StartsWith(_keyPrefixAuto))
                    {
                        _autoMode.UpdateValue(key, value);  // update auto variable
                        return (true);
                    }

                    // nothing we care about
                    return (false);

                case DashboardNetworkTable.Connected:
                    // if here, then the robot is there and we should have
                    // connected to the network tables after the robot got there.
                    // so now we need to send down the things that come from the
                    // dashboard so the robot's network table has that info.
                    // if he had done it before the robot's network table was
                    // there, then our setting would only be local and then it
                    // would get wiped out by the robot table.

                    // set default auto mode. MUST do after network table is setup.
                    _autoMode.Connected();
                    if (_panelConnected != null) { _panelConnected.BackColor = Color.Green; }
                    if (CameraFeed != null) { CameraFeed.Connected(); }
                    if (CameraFeedLimeLight != null) { CameraFeedLimeLight.Connected(); }

                    // tell the robot who we are
                    SetPcIpAddress();
                    break;

                case DashboardNetworkTable.Disconnected:
                    if (_panelConnected != null) { _panelConnected.BackColor = Color.Red; }
                    if (CameraFeed != null) { CameraFeed.Disconnected(); }
                    if (CameraFeedLimeLight != null) { CameraFeedLimeLight.Disconnected(); }
                    break;

                case _autoModeKey: // value robot has
                    if (_labelAutoModeValue != null)
                    {
                        _labelAutoModeValue.Text = value;
                    }
                    break;

                case "thunderdashboard_drivearcade":
                    // sees if robot drive is in arcade mode (when Tank is the default)
                    if (DriveType != null)
                    {
                        if (value == "1")
                        {
                            DriveType.Visible = !DriveTypeDefaultIsArcade;
                        }
                        else
                        {
                            DriveType.Visible = DriveTypeDefaultIsArcade;
                        }
                    }
                    break;

                case "thunderdashboard_drivereverse":
                    // sees if robot drive is reversed
                    if (RobotDirection != null)
                    {
                        if (value == "1")
                        {
                            RobotDirection.Visible = true;
                        }
                        else
                        {
                            RobotDirection.Visible = false;
                        }
                    }
                    break;

                case "thunderdashboard_frontcamera":
                    if (_labelCameraSide != null)
                    {
                        if (value == "1")
                        {
                            _labelCameraSide.Text = "F\r\nR\r\nO\r\nN\r\nT";
                        }
                        else
                        {
                            _labelCameraSide.Text = "R\r\nE\r\nA\r\nR";
                        }
                    }

                    // if we have two feeds, then show the proper one
                    if ((CameraFeed != null) && (CameraFeedLimeLight != null))
                    {
                        if (value == "1")
                        {
                            CameraFeed.Visible = true;
                            CameraFeedLimeLight.Visible = false;
                            if (!CameraBothStreamsActive)
                            {
                                StartCameras(true, false);
                            }
                        }
                        else
                        {
                            CameraFeed.Visible = false;
                            CameraFeedLimeLight.Visible = true;
                            if (!CameraBothStreamsActive)
                            {
                                StartCameras(false, true);
                            }
                        }
                    }
                    break;

                case "vision_Start":
                    if (CameraFeed == null)
                    {
                        break;      // no camera control
                    }
                    if (value != "true")
                    {
                        DashboardNetworkTable.PutNumber("vision_Result", (double)0);    // make robot doesn't read anything
                        CameraFeed.VisionStop();
                    }
                    else
                    {
                        CameraFeed.VisionStart();
                    }
                    break;
            }
            return (false);
        }

        public virtual bool VisionProcess()
        {
            return (false);
        }

        // reset button was pressed
        public virtual void buttonReset_Click(object sender, EventArgs e)
        {
            if (_panelConnected != null) { _panelConnected.BackColor = Color.Yellow; }
            DashboardNetworkTable.Restart();
            if ((CameraFeed != null) && (CameraBothStreamsActive || CameraFeed.Visible)) { CameraFeed.Restart(); }
            if ((CameraFeedLimeLight != null) && (CameraBothStreamsActive || CameraFeedLimeLight.Visible)) { CameraFeedLimeLight.Restart(); }
        }

        // this deals with the complexity of whether we can have both streams open at the same time or not.
        // it also deals with whether we actually have both controls or not
        private void StartCameras(bool startCamera, bool startCameraLimeLight)
        {
            // handle the main camera
            if (CameraFeed != null)
            {
                if (startCamera)
                {
                    CameraFeed.Start(RobotIpAddress, CameraPort, StatusTextBox, CameraFlip, CameraVisionDelegate, CameraCanChangeSettings, CameraStream, "robot", CameraImage);
                }
                else
                {
                    CameraFeed.Stop();
                }
            }

            // handle the LimeLight
            if (CameraFeedLimeLight != null)
            {
                if (startCameraLimeLight)
                {
                    CameraFeedLimeLight.Start(CameraIpLimeLight, CameraPortLimeLight, StatusTextBox, false, null, false, CameraStreamLimeLight, "LimeLight", CameraImageLimeLight);
                }
                else
                {
                    CameraFeedLimeLight.Stop();
                }
            }
        }

        // put PC's IP Address in the network tables
        private void SetPcIpAddress()
        {
            string ip;

            // first try to find the one that matches our team number
            ip = FindIpAddress("10." + _teamNumber.Substring(0, 2) + "." + _teamNumber.Substring(2, 2));
            if (ip == null)
            {
                // didn't find our team network, so take the first one found
                ip = FindIpAddress(null);
            }
            if (ip != null)
            {
                // found an IP to use, so put it in the network table
                DashboardNetworkTable.PutString("thunderdashboard_ip", ip);
            }
        }

        // get our PC's network address for the robot's
        private string FindIpAddress(string startingIp)
        {
            // look at each network card on the PC
            foreach (NetworkInterface n in NetworkInterface.GetAllNetworkInterfaces())
            {
                // make sure the network is connected
                if (n.OperationalStatus != OperationalStatus.Up)
                {
                    continue;       // netowrk is not connected
                }

                // look at each address for the NIC
                foreach (UnicastIPAddressInformation ip in n.GetIPProperties().UnicastAddresses)
                {
                    // make sure it is IPv4
                    if (ip.Address.AddressFamily != System.Net.Sockets.AddressFamily.InterNetwork)
                    {
                        continue;   // not IPv4
                    }

                    // is this the one connected to our robot?
                    if (string.IsNullOrEmpty(startingIp) || ip.Address.ToString().StartsWith(startingIp))
                    {
                        return (ip.Address.ToString());   // this is our robot's network, so this is our PC's address
                    }
                }
            }

            // did not find our robot's network
            return (null);
        }

        // update the driver type label based on what is the default
        private void UpdateDriveType()
        {
            if (DriveType == null)
            {
                return;
            }

            // we only show something when it isn't the default. so the text is the
            // opposite of the default
            string text;
            text = (DriveTypeDefaultIsArcade ? "T\r\nA\r\nN\r\nK" : "A\r\nR\r\nC\r\nA\r\nD\r\nE");
            text += "\r\n\r\nD\r\nR\r\nI\r\nV\r\nE";
            DriveType.Text = text;
        }


        private const string _autoModeKey = "Auto_Mode";
        private const string _keyPrefixAuto = "thunderdashboard_auto_";
        private AutoMode _autoMode = new AutoMode();

        // vision
        protected int _visionCounter = 1;                 // increment for each processed image
    }
}
