﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ThunderDashboard
{
    class MeterControl : Panel
    {
        // how to draw the text on the meter
        public enum DrawType
        {
            None,       // do not draw anything
            Decimal     // draw as a decimal value
        };

        // meter's value
        public string Value
        {
            get { return _value.ToString(); }
            set
            {
                // if string is invalide, then set to minimum
                if (string.IsNullOrEmpty(value))
                {
                    SetValue(_minValue);
                }
                else
                {
                    try
                    {
                        SetValue(Convert.ToDouble(value));
                    }
                    catch
                    {
                        SetValue(_minValue);
                    }
                }
            }
        }

        // meter's target value
        public string ValueTarget
        {
            get { return _valueTarget.ToString(); }
            set
            {
                // if string is invalide, then set to minimum
                if (string.IsNullOrEmpty(value))
                {
                    SetValueTarget(_minValue);
                }
                else
                {
                    try
                    {
                        SetValueTarget(Convert.ToDouble(value));
                    }
                    catch
                    {
                        SetValueTarget(_minValue);
                    }
                }
            }
        }

        // set the decimal places for the number within the gauge
        public int ThunderDecimalPlaces
        {
            get { return (_decimalPlaces); }
            set
            {
                if (value != _decimalPlaces)
                {
                    _decimalPlaces = value;
                    Redraw();
                }
            }
        }

        // is the meter gauge drawn horizontally
        public bool ThunderMeterHorizontal
        {
            get { return _meterHorizontal; }
            set { _meterHorizontal = value; }
        }

        // is the meter drawn left-to-right (i.e. horizontal)
        public bool ThunderMeterLeftToRight
        {
            get { return _meterLeftToRight; }
            set { _meterLeftToRight = value; }
        }

        // how to draw the value on the gauge, if at all
        public DrawType ThunderMeterDrawType
        {
            get { return _meterDrawType; }
            set { _meterDrawType = value; }
        }

        // set the color for the meter (set to NULL to use default color)
        public void Color(System.Drawing.Color color)
		{
			// resetting color?
			if (color == null)
			{
				color = _colorDefault;
			}

			// if color changed, then save it and re-draw the meter
			if (color != _color)
			{
				_color = color;
				Redraw();
			}
		}

        // set the target color for the meter (set to NULL to use default color)
        public void ColorTarget(System.Drawing.Color color)
        {
            // resetting color?
            if (color == null)
            {
                color = _colorDefaultTarget;
            }

            // if color changed, then save it and re-draw the meter
            if (color != _colorTarget)
            {
                _colorTarget = color;
                Redraw();
            }
        }

        // will reset the color to the default
        public void ColorReset()
        {
            Color(_colorDefault);
            ColorTarget(_colorDefaultTarget);
        }

        // set the Min and Max values that could ever be sent (i.e. bottom and top value for the guage)
        public void MinMax(double min, double max)
		{
			double value = _value;

			if (min > max)
			{
				min = max;
			}
			if (_value > max)
			{
				_value = max;
			}
			if (_value < min)
			{
				_value = min;
			}
			if ((min != _minValue) || (max != _maxValue) || (value != _value))
			{
				_minValue = min;
				_maxValue = max;
				_value = value;
				Redraw();
			}
		}

        protected override void OnClick(EventArgs e)
        {
            base.OnClick(e);
            //int xx = Height - PointToClient(Cursor.Position).Y;
            //Value = ((PointToClient(Cursor.Position).Y - _minValue) * Height / (_maxValue - _minValue)).ToString();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            DrawFillRectangle();
        }

		// force control to be redrawn
		private void Redraw()
		{
			// This repaints the control. Per MSDN: Calling the Invalidate method does not force a synchronous paint;
			// to force a synchronous paint, call the Update method after calling the Invalidate method.
			this.Invalidate();
			this.Update();
		}

		private void DrawFillRectangle()
        {
            // Draws the green rectangle indicating the value set.
            System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(_color);
            System.Drawing.SolidBrush myBrushTarget = new System.Drawing.SolidBrush(_colorTarget);
            System.Drawing.Graphics formGraphics;
            formGraphics = this.CreateGraphics();

            int width = 0;
            int height = 0;
            int x = 0;
            int y = 0;
            double percent = ((double)_value - (double)_minValue) / ((double)_maxValue - (double)_minValue);
            int widthTarget = 0;
            int heightTarget = 0;
            int xTarget = 0;
            int yTarget = 0;
            double percentTarget = ((double)_valueTarget - (double)_minValue) / ((double)_maxValue - (double)_minValue);

            if (ThunderMeterHorizontal)
            {
                height = this.Height;
                heightTarget = this.Height;
                width = (int)(this.Width * percent); // Width should be a percentage of control
                widthTarget = (int)(this.Width * percentTarget);

                // if we have a target, then adjust if it is different
                // if target is more than current: show all of Green current, then Red from there to target
                // if target is less than current: show all of Red target, then Green from there to current
                if ((_valueTarget >= 0) && (_valueTarget != _value))
                {
                    if (_valueTarget > _value)
                    {
                        xTarget = width;
                        widthTarget = widthTarget - width;
                    }
                    else
                    {
                        x = widthTarget;
                        width = width - widthTarget;
                    }
                }

                if (!ThunderMeterLeftToRight)
                {
                    x = this.Width - width - x; // Need to change the x position of the rectangle to be drawn
                    xTarget = this.Width - widthTarget - xTarget;
                }
            }
            else // vertical layout
            {
                width = this.Width;
                widthTarget = this.Width;
                height = (int)(this.Height * percent); // Height should be a percentage of control;
                heightTarget = (int)(this.Height * percentTarget);

                y = this.Height - height;
                yTarget = this.Height - heightTarget;

                // if we have a target, then adjust if it is different
                // if target is more than current: show all of Green current, then Red from there to target
                // if target is less than current: show all of Red target, then Green from there to current
                if ((_valueTarget >= 0) && (_valueTarget != _value))
                {
                    if (_valueTarget > _value)
                    {
                        heightTarget = y - yTarget;
                    }
                    else
                    {
                        height = yTarget - y;
                    }
                }
            }

            formGraphics.FillRectangle(myBrush, new Rectangle(x, y, width, height));
            if ((_valueTarget >= 0) && (_valueTarget != _value))
            {
                formGraphics.FillRectangle(myBrushTarget, new Rectangle(xTarget, yTarget, widthTarget, heightTarget));
            }
            myBrush.Dispose();
            myBrushTarget.Dispose();

            // draw text based on the type
            if (ThunderMeterDrawType != DrawType.None)
            {
                StringFormat f = new StringFormat();
                f.LineAlignment = StringAlignment.Center;
                f.Alignment = StringAlignment.Center;

                double d = 100.0 * percent;
                string s = string.Format("{0:N" + _decimalPlaces + "}", d);
                formGraphics.DrawString(s, new Font("Arial", 18), Brushes.Black, this.ClientRectangle, f);
            }

            formGraphics.Dispose();
        }

        // make sure the value stays within the Min/Max range
        private void SetValue(double value)
        {
            // make sure value is within range
            if (value > _maxValue)
            {
                value = _maxValue;
            }
            if (value < _minValue)
            {
                value = _minValue;
            }

            // if the value has changed, then save it and redraw meter
            if (_value != value)
            {
                _value = value;
                Redraw();
            }
        }

        // make sure the target value stays within the Min/Max range
        private void SetValueTarget(double value)
        {
            // make sure value is within range
            if (value > _maxValue)
            {
                value = _maxValue;
            }
            if (value < _minValue)
            {
                value = _minValue;
            }

            // if the value has changed, then save it and redraw meter
            if (_valueTarget != value)
            {
                _valueTarget = value;
                Redraw();
            }
        }

        // color for the meter
        private static System.Drawing.Color _colorDefault = System.Drawing.Color.Lime;
        private static System.Drawing.Color _colorDefaultTarget = System.Drawing.Color.Red;
        private System.Drawing.Color _color = _colorDefault;
        private System.Drawing.Color _colorTarget = _colorDefaultTarget;

        // how to draw the meter
        private bool _meterHorizontal = true;
		private bool _meterLeftToRight = true;
        private DrawType _meterDrawType = DrawType.Decimal;

        // current value
        private double _value = 0;
        private double _valueTarget = -1;
        private double _maxValue = 100;
		private double _minValue = 0;
        private int _decimalPlaces = 0;
	}
}
